# PowerShell script to start JBoss EAP 8.0

# Read configuration from config.json
$configPath = Join-Path $PSScriptRoot "config.json"
if (-not (Test-Path $configPath)) {
    Write-Host "Error: config.json not found at $configPath" -ForegroundColor Red
    exit 1
}

$config = Get-Content $configPath -Raw | ConvertFrom-Json

# Configuration from config.json
$JBOSS_HOME = $config.jboss.jbossHome
$JAVA_HOME = $config.jboss.javaHome
$CONFIG_FILE = $config.jboss.configFile

# Function to check if JBoss is running
function Test-JBossRunning {
    $jbossProcess = Get-CimInstance Win32_Process -Filter "Name = 'java.exe'" | 
                    Where-Object { $_.CommandLine -like "*jboss.home.dir=$JBOSS_HOME*" }
    return $null -ne $jbossProcess
}

# Check if Java exists
if (-not (Test-Path $JAVA_HOME)) {
    Write-Host "Error: Java not found at $JAVA_HOME" -ForegroundColor Red
    exit 1
}

# Check if JBoss directory exists
if (-not (Test-Path $JBOSS_HOME)) {
    Write-Host "Error: JBoss directory not found at $JBOSS_HOME" -ForegroundColor Red
    exit 1
}

# Copy configuration file from local config directory
$localConfigDir = Join-Path $PSScriptRoot "config"
$localConfigFile = Join-Path $localConfigDir (Split-Path $CONFIG_FILE -Leaf)
$jbossConfigDir = Join-Path $JBOSS_HOME "standalone\configuration"

if (Test-Path $localConfigFile) {
    Write-Host "Copying configuration file from local config directory..." -ForegroundColor Yellow
    Write-Host "From: $localConfigFile" -ForegroundColor Yellow
    Write-Host "To: $jbossConfigDir" -ForegroundColor Yellow
    
    # Create backup of existing config if it exists
    $targetConfigFile = Join-Path $jbossConfigDir (Split-Path $CONFIG_FILE -Leaf)
    if (Test-Path $targetConfigFile) {
        $backupFile = "$targetConfigFile.backup"
        Write-Host "Creating backup of existing config: $backupFile" -ForegroundColor Yellow
        Copy-Item $targetConfigFile $backupFile -Force
    }
    
    # Copy the new config file
    Copy-Item $localConfigFile $jbossConfigDir -Force
} else {
    Write-Host "Warning: Local config file not found at $localConfigFile" -ForegroundColor Yellow
    Write-Host "Using existing JBoss configuration" -ForegroundColor Yellow
}

# Check if configuration file exists in JBoss
$configFile = Join-Path $jbossConfigDir (Split-Path $CONFIG_FILE -Leaf)
if (-not (Test-Path $configFile)) {
    Write-Host "Error: Configuration file not found at $configFile" -ForegroundColor Red
    exit 1
}

# Set environment variables
$env:JAVA_HOME = $JAVA_HOME
$env:JBOSS_HOME = $JBOSS_HOME
$env:PATH = "$JAVA_HOME\bin;$env:PATH"
$env:NOPAUSE = "true"
# Remove this as it might affect logging
# $env:LAUNCH_JBOSS_IN_BACKGROUND = "true"

# Check if JBoss is already running
if (Test-JBossRunning) {
    Write-Host "JBoss is already running!" -ForegroundColor Yellow
    exit 0
}

Write-Host "Starting JBoss EAP 8.0..." -ForegroundColor Green
Write-Host "Using Java from: $JAVA_HOME" -ForegroundColor Yellow
Write-Host "JBoss Home: $JBOSS_HOME" -ForegroundColor Yellow
Write-Host "Using configuration: $CONFIG_FILE" -ForegroundColor Yellow

# Start JBoss in standalone mode
$startScript = Join-Path $JBOSS_HOME "bin\standalone.bat"
$jvmOptions = @(
    "-DIDServer=GS",
    "-Dbtf.PathToParse=C:\Dev2012\advc0\in",
    "-Djava.awt.headless=true",
    "-DFilOptimize=true",
    "-DMTH_VALID_JB=internal",
    "-DskQueueElab=GS",
    "-DskQueueWrite=GS",
    "-DmtSession=GS",
    "-agentlib:jdwp=transport=dt_socket,address=8787,server=y,suspend=n"
)

# Add JVM options to JAVA_OPTS
$env:JAVA_OPTS = "$($jvmOptions -join ' ')"

Write-Host "Starting JBoss with custom JVM options..." -ForegroundColor Green
Write-Host "JVM Options: $env:JAVA_OPTS" -ForegroundColor Yellow
Write-Host "Remote debugging enabled on port 8787" -ForegroundColor Cyan

# Create the command line arguments
$cmdArgs = "-c $CONFIG_FILE --debug -b 0.0.0.0 -bmanagement 0.0.0.0"

# Start JBoss - Modified to use Start-Process with RedirectStandardOutput
$logFile = Join-Path $JBOSS_HOME "standalone\log\server-info.log"
$processStartInfo = @{
    FilePath = $startScript
    ArgumentList = $cmdArgs
    RedirectStandardOutput = $logFile
    RedirectStandardError = Join-Path $JBOSS_HOME "standalone\log\server_error.log"
    WorkingDirectory = Join-Path $JBOSS_HOME "bin"
    NoNewWindow = $true
    PassThru = $true
}

Write-Host "Starting JBoss and redirecting output to $logFile" -ForegroundColor Yellow
$jbossProcess = Start-Process @processStartInfo

# Wait a few seconds to check if the process started successfully
Start-Sleep -Seconds 10

if (Test-JBossRunning) {
    Write-Host "JBoss started successfully!" -ForegroundColor Green
    Write-Host "Admin console will be available at: http://localhost:9990" -ForegroundColor Yellow
    Write-Host "Remote debugging is enabled on port 8787" -ForegroundColor Cyan
    Write-Host "Check JBoss logs at: $logFile" -ForegroundColor Yellow
} else {
    Write-Host "Error: JBoss failed to start. Check the logs at: $logFile" -ForegroundColor Red
}
