# PowerShell script to stop JBoss EAP 8.0

# Read configuration from config.json
$configPath = Join-Path $PSScriptRoot "config.json"
if (-not (Test-Path $configPath)) {
    Write-Host "Error: config.json not found at $configPath" -ForegroundColor Red
    exit 1
}

$config = Get-Content $configPath -Raw | ConvertFrom-Json

# Configuration from config.json
$JBOSS_HOME = $config.jboss.jbossHome

Write-Host "Stopping JBoss EAP 8.0..." -ForegroundColor Yellow
Write-Host "JBoss Home: $JBOSS_HOME" -ForegroundColor Yellow

# Find and stop JBoss process
$jbossProcess = Get-CimInstance Win32_Process -Filter "Name = 'java.exe'" | 
                Where-Object { $_.CommandLine -like "*jboss.home.dir=$JBOSS_HOME*" }

if ($jbossProcess) {
    Write-Host "Found JBoss process (PID: $($jbossProcess.ProcessId))" -ForegroundColor Yellow
    
    # Try graceful shutdown first
    Write-Host "Attempting graceful shutdown..." -ForegroundColor Yellow
    $jbossCli = Join-Path $JBOSS_HOME "bin\jboss-cli.bat"
    $cliCommand = "--connect --command=:shutdown"
    
    try {
        Start-Process -FilePath $jbossCli -ArgumentList $cliCommand -Wait -NoNewWindow
        Start-Sleep -Seconds 5  # Give it time for graceful shutdown
        
        # Check if process is still running
        $processStillRunning = Get-Process -Id $jbossProcess.ProcessId -ErrorAction SilentlyContinue
        
        if ($processStillRunning) {
            Write-Host "Graceful shutdown failed, forcing process termination..." -ForegroundColor Yellow
            Stop-Process -Id $jbossProcess.ProcessId -Force
        }
    }
    catch {
        Write-Host "Graceful shutdown failed, forcing process termination..." -ForegroundColor Yellow
        Stop-Process -Id $jbossProcess.ProcessId -Force
    }
    
    Write-Host "JBoss process stopped" -ForegroundColor Green
} else {
    Write-Host "No JBoss process found running" -ForegroundColor Yellow
}

# Wait a moment to ensure process is fully stopped
Start-Sleep -Seconds 2

# Verify JBoss is stopped
$jbossStillRunning = Get-CimInstance Win32_Process -Filter "Name = 'java.exe'" | 
                     Where-Object { $_.CommandLine -like "*jboss.home.dir=$JBOSS_HOME*" }

if ($jbossStillRunning) {
    Write-Host "Warning: JBoss process is still running!" -ForegroundColor Red
} else {
    Write-Host "Confirmed: JBoss is fully stopped" -ForegroundColor Green
}
